@@ -45,9 +45,9 @@ module Hledger.Data.Dates (
   spansSpan,
   spanIntersect,
   spansIntersect,
+  spanDefaultsFrom,
   spanUnion,
   spansUnion,
-  orDatesFrom,
   smartdate,
   splitSpan,
   fixSmartDate,
@@ -78,17 +78,19 @@ import Hledger.Data.Types
 import Hledger.Utils
 
 
+-- Help ppShow parse and line-wrap DateSpans better in debug output.
+instance Show DateSpan where
+    show (DateSpan s1 s2) = "DateSpan \"" ++ show s1 ++ "\" \"" ++ show s2 ++ "\""
+
 showDate :: Day -> String
-showDate = formatTime defaultTimeLocale "%C%y/%m/%d"
+showDate = formatTime defaultTimeLocale "%0C%y/%m/%d"
 
 showDateSpan (DateSpan from to) =
   concat
-    [maybe "" showdate from
+    [maybe "" showDate from
     ,"-"
-    ,maybe "" (showdate . prevday) to
+    ,maybe "" (showDate . prevday) to
     ]
-  where
-    showdate = formatTime defaultTimeLocale "%C%y/%m/%d"
 
 -- | Get the current local date.
 getCurrentDay :: IO Day
@@ -117,11 +119,14 @@ spanStart (DateSpan d _) = d
 spanEnd :: DateSpan -> Maybe Day
 spanEnd (DateSpan _ d) = d
 
+-- might be useful later: http://en.wikipedia.org/wiki/Allen%27s_interval_algebra 
+
 -- | Get overall span enclosing multiple sequentially ordered spans.
 spansSpan :: [DateSpan] -> DateSpan
 spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Nothing spanEnd $ lastMay spans)
 
--- | Split a DateSpan into one or more consecutive spans at the specified interval.
+-- | Split a DateSpan into one or more consecutive whole spans of the specified length which enclose it.
+-- If no interval is specified, the original span is returned.
 splitSpan :: Interval -> DateSpan -> [DateSpan]
 splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
 splitSpan NoInterval     s = [s]
@@ -166,14 +171,6 @@ spanContainsDate (DateSpan Nothing (Just e))  d = d < e
 spanContainsDate (DateSpan (Just b) Nothing)  d = d >= b
 spanContainsDate (DateSpan (Just b) (Just e)) d = d >= b && d < e
     
--- | Combine two datespans, filling any unspecified dates in the first
--- with dates from the second. Not a clip operation, just uses the
--- second's start/end dates as defaults when the first does not
--- specify them.
-orDatesFrom (DateSpan a1 b1) (DateSpan a2 b2) = DateSpan a b
-    where a = if isJust a1 then a1 else a2
-          b = if isJust b1 then b1 else b2
-
 -- | Calculate the intersection of a number of datespans.
 spansIntersect [] = nulldatespan
 spansIntersect [d] = d
@@ -185,6 +182,12 @@ spanIntersect (DateSpan b1 e1) (DateSpan b2 e2) = DateSpan b e
       b = latest b1 b2
       e = earliest e1 e2
 
+-- | Fill any unspecified dates in the first span with the dates from
+-- the second one. Sort of a one-way spanIntersect.
+spanDefaultsFrom (DateSpan a1 b1) (DateSpan a2 b2) = DateSpan a b
+    where a = if isJust a1 then a1 else a2
+          b = if isJust b1 then b1 else b2
+
 -- | Calculate the union of a number of datespans.
 spansUnion [] = nulldatespan
 spansUnion [d] = d
@@ -675,11 +678,11 @@ tests_Hledger_Data_Dates = TestList
   ,"period expressions" ~: do
     let todaysdate = parsedate "2008/11/26"
     let str `gives` result = show (parsewith (periodexpr todaysdate) str) `is` ("Right " ++ result)
-    "from aug to oct"           `gives` "(NoInterval,DateSpan (Just 2008-08-01) (Just 2008-10-01))"
-    "aug to oct"                `gives` "(NoInterval,DateSpan (Just 2008-08-01) (Just 2008-10-01))"
-    "every 3 days in aug"       `gives` "(Days 3,DateSpan (Just 2008-08-01) (Just 2008-09-01))"
-    "daily from aug"            `gives` "(Days 1,DateSpan (Just 2008-08-01) Nothing)"
-    "every week to 2009"        `gives` "(Weeks 1,DateSpan Nothing (Just 2009-01-01))"
+    "from aug to oct"           `gives` "(NoInterval,DateSpan \"Just 2008-08-01\" \"Just 2008-10-01\")"
+    "aug to oct"                `gives` "(NoInterval,DateSpan \"Just 2008-08-01\" \"Just 2008-10-01\")"
+    "every 3 days in aug"       `gives` "(Days 3,DateSpan \"Just 2008-08-01\" \"Just 2008-09-01\")"
+    "daily from aug"            `gives` "(Days 1,DateSpan \"Just 2008-08-01\" \"Nothing\")"
+    "every week to 2009"        `gives` "(Weeks 1,DateSpan \"Nothing\" \"Just 2009-01-01\")"
 
   ,"splitSpan" ~: do
     let gives (interval, span) = (splitSpan interval span `is`)
@@ -727,6 +730,7 @@ tests_Hledger_Data_Dates = TestList
 
   ,"fixSmartDateStr" ~: do
     let gives = is . fixSmartDateStr (parsedate "2008/11/26")
+    "0000-01-01"   `gives` "0000/01/01"
     "1999-12-02"   `gives` "1999/12/02"
     "1999.12.02"   `gives` "1999/12/02"
     "1999/3/2"     `gives` "1999/03/02"
