@@ -26,7 +26,9 @@ module Hledger.Data.Posting (
   postingDate,
   postingDate2,
   isPostingInDateSpan,
+  isPostingInDateSpan',
   postingsDateSpan,
+  postingsDateSpan',
   -- * account name operations
   accountNamesFromPostings,
   accountNamePostingType,
@@ -145,11 +147,11 @@ postingCleared p = if pstatus p
 
 -- | Tags for this posting including any inherited from its parent transaction.
 postingAllTags :: Posting -> [Tag]
-postingAllTags p = ptags p ++ maybe [] transactionAllTags (ptransaction p)
+postingAllTags p = ptags p ++ maybe [] ttags (ptransaction p)
 
--- | Tags for this transaction including any inherited from above, when that is implemented.
+-- | Tags for this transaction including any from its postings.
 transactionAllTags :: Transaction -> [Tag]
-transactionAllTags t = ttags t
+transactionAllTags t = ttags t ++ concatMap ptags (tpostings t)
 
 -- Get the other postings from this posting's transaction.
 relatedPostings :: Posting -> [Posting]
@@ -160,6 +162,11 @@ relatedPostings _ = []
 isPostingInDateSpan :: DateSpan -> Posting -> Bool
 isPostingInDateSpan s = spanContainsDate s . postingDate
 
+-- --date2-sensitive version, separate for now to avoid disturbing multiBalanceReport.
+isPostingInDateSpan' :: WhichDate -> DateSpan -> Posting -> Bool
+isPostingInDateSpan' PrimaryDate   s = spanContainsDate s . postingDate
+isPostingInDateSpan' SecondaryDate s = spanContainsDate s . postingDate2
+
 isEmptyPosting :: Posting -> Bool
 isEmptyPosting = isZeroMixedAmount . pamount
 
@@ -170,6 +177,14 @@ postingsDateSpan [] = DateSpan Nothing Nothing
 postingsDateSpan ps = DateSpan (Just $ postingDate $ head ps') (Just $ addDays 1 $ postingDate $ last ps')
     where ps' = sortBy (comparing postingDate) ps
 
+-- --date2-sensitive version, as above.
+postingsDateSpan' :: WhichDate -> [Posting] -> DateSpan
+postingsDateSpan' _  [] = DateSpan Nothing Nothing
+postingsDateSpan' wd ps = DateSpan (Just $ postingdate $ head ps') (Just $ addDays 1 $ postingdate $ last ps')
+    where
+      ps' = sortBy (comparing postingdate) ps
+      postingdate = if wd == PrimaryDate then postingDate else postingDate2
+
 -- AccountName stuff that depends on PostingType
 
 accountNamePostingType :: AccountName -> PostingType
