@@ -21,10 +21,12 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           ---- all of this one:
                           module Hledger.Utils,
                           Debug.Trace.trace,
-                          module Data.PPrint,
+                          -- module Data.PPrint,
                           -- module Hledger.Utils.UTF8IOCompat
                           SystemString,fromSystemString,toSystemString,error',userError',
+#if __GLASGOW_HASKELL__ >= 704
                           ppShow
+#endif
                           -- the rest need to be done in each module I think
                           )
 where
@@ -32,11 +34,10 @@ import Control.Monad (liftM, when)
 import Control.Monad.Error (MonadIO)
 import Control.Monad.IO.Class (liftIO)
 import Data.Char
-import Data.Data
 import Data.List
 import qualified Data.Map as M
 import Data.Maybe
-import Data.PPrint
+-- import Data.PPrint
 import Data.Time.Clock
 import Data.Time.LocalTime
 import Data.Tree
@@ -51,15 +52,22 @@ import System.IO.Unsafe (unsafePerformIO)
 import Test.HUnit
 import Text.ParserCombinators.Parsec
 import Text.Printf
-import Text.Regex
+import Text.Regex.TDFA
 import Text.RegexPR
-import Text.Show.Pretty
 -- import qualified Data.Map as Map
 -- 
 -- import Prelude hiding (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
 -- import Hledger.Utils.UTF8IOCompat   (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
 import Hledger.Utils.UTF8IOCompat (SystemString,fromSystemString,toSystemString,error',userError')
 
+#if __GLASGOW_HASKELL__ >= 704
+import Text.Show.Pretty (ppShow)
+#else
+-- the required pretty-show version requires GHC >= 7.4
+ppShow :: Show a => a -> String
+ppShow = show
+#endif
+
 -- strings
 
 lowercase = map toLower
@@ -83,7 +91,7 @@ underline s = s' ++ replicate (length s) '-' ++ "\n"
             | last s == '\n' = s
             | otherwise = s ++ "\n"
 
--- | Wrap a string in single quotes, and \-prefix any embedded single
+-- | Wrap a string in double quotes, and \-prefix any embedded single
 -- quotes, if it contains whitespace and is not already single- or
 -- double-quoted.
 quoteIfSpaced :: String -> String
@@ -91,6 +99,22 @@ quoteIfSpaced s | isSingleQuoted s || isDoubleQuoted s = s
                 | not $ any (`elem` s) whitespacechars = s
                 | otherwise = "'"++escapeSingleQuotes s++"'"
 
+-- | Double-quote this string if it contains whitespace, single quotes
+-- or double-quotes, escaping the quotes as needed.
+quoteIfNeeded s | any (`elem` s) (quotechars++whitespacechars) = "\"" ++ escapeDoubleQuotes s ++ "\""
+                | otherwise = s
+
+-- | Single-quote this string if it contains whitespace or double-quotes.
+-- No good for strings containing single quotes.
+singleQuoteIfNeeded s | any (`elem` s) whitespacechars = "'"++s++"'"
+                      | otherwise = s
+
+quotechars      = "'\""
+whitespacechars = " \t\n\r"
+
+escapeDoubleQuotes :: String -> String
+escapeDoubleQuotes = regexReplace "\"" "\""
+
 escapeSingleQuotes :: String -> String
 escapeSingleQuotes = regexReplace "'" "\'"
 
@@ -103,21 +127,16 @@ words' :: String -> [String]
 words' "" = []
 words' s  = map stripquotes $ fromparse $ parsewith p s
     where
-      p = do ss <- (quotedPattern <|> pattern) `sepBy` many1 spacenonewline
+      p = do ss <- (singleQuotedPattern <|> doubleQuotedPattern <|> pattern) `sepBy` many1 spacenonewline
              -- eof
              return ss
       pattern = many (noneOf whitespacechars)
-      quotedPattern = between (oneOf "'\"") (oneOf "'\"") $ many $ noneOf "'\""
+      singleQuotedPattern = between (char '\'') (char '\'') (many $ noneOf "'")
+      doubleQuotedPattern = between (char '"') (char '"') (many $ noneOf "\"")
 
 -- | Quote-aware version of unwords - single-quote strings which contain whitespace
 unwords' :: [String] -> String
-unwords' = unwords . map singleQuoteIfNeeded
-
--- | Single-quote this string if it contains whitespace or double-quotes
-singleQuoteIfNeeded s | any (`elem` s) whitespacechars = "'"++s++"'"
-                      | otherwise = s
-
-whitespacechars = " \t\n\r"
+unwords' = unwords . map quoteIfNeeded
 
 -- | Strip one matching pair of single or double quotes on the ends of a string.
 stripquotes :: String -> String
@@ -207,6 +226,23 @@ fitto w h s = intercalate "\n" $ take h $ rows ++ repeat blankline
       fit w = take w . (++ repeat ' ')
       blankline = replicate w ' '
 
+-- tuples
+
+first3  (x,_,_) = x
+second3 (_,x,_) = x
+third3  (_,_,x) = x
+
+first4  (x,_,_,_) = x
+second4 (_,x,_,_) = x
+third4  (_,_,x,_) = x
+fourth4 (_,_,_,x) = x
+
+first5  (x,_,_,_,_) = x
+second5 (_,x,_,_,_) = x
+third5  (_,_,x,_,_) = x
+fourth5 (_,_,_,x,_) = x
+fifth5  (_,_,_,_,x) = x
+
 -- math
 
 difforzero :: (Num a, Ord a) => a -> a -> a
@@ -246,10 +282,10 @@ regexSplit = splitRegexPR
 
 -- regex-compat (regex-posix) functions that perform better than regexpr.
 regexMatchesRegexCompat :: String -> String -> Bool
-regexMatchesRegexCompat r = isJust . matchRegex (mkRegex r)
+regexMatchesRegexCompat = flip (=~)
 
 regexMatchesCIRegexCompat :: String -> String -> Bool
-regexMatchesCIRegexCompat r = isJust . matchRegex (mkRegexWithOpts r True False)
+regexMatchesCIRegexCompat r = match (makeRegexOpts defaultCompOpt { multiline = True, caseSensitive = False, newSyntax = True } defaultExecOpt r)
 
 -- lists
 
@@ -398,7 +434,24 @@ debugLevel = case snd $ break (=="--debug") args of
 -- | Print a message and a showable value to the console if the global
 -- debug level is non-zero.  Uses unsafePerformIO.
 dbg :: Show a => String -> a -> a
-dbg = dbgppshow 1
+dbg = dbg1
+
+dbg0 :: Show a => String -> a -> a
+dbg0 = dbgAt 0
+
+dbg1 :: Show a => String -> a -> a
+dbg1 = dbgAt 1
+
+dbg2 :: Show a => String -> a -> a
+dbg2 = dbgAt 2
+
+-- | Print a message and a showable value to the console if the global
+-- debug level is at or above the specified level.  Uses unsafePerformIO.
+dbgAt :: Show a => Int -> String -> a -> a
+dbgAt lvl = dbgppshow lvl
+
+dbgAtM :: Show a => Int -> String -> a -> IO ()
+dbgAtM lvl lbl x = dbgAt lvl lbl x `seq` return ()
 
 -- | Print a showable value to the console, with a message, if the
 -- debug level is at or above the specified level (uses
@@ -415,21 +468,27 @@ dbgshow level
 -- Values are displayed with ppShow, each field/constructor on its own line.
 dbgppshow :: Show a => Int -> String -> a -> a
 dbgppshow level
-    | debugLevel >= level = \s -> traceWith (((s++": ")++) . ppShow)
-    | otherwise           = flip const
+    | debugLevel < level = flip const
+    | otherwise = \s a -> let p = ppShow a
+                              ls = lines p
+                              nlorspace | length ls > 1 = "\n"
+                                        | otherwise     = " " ++ take (10 - length s) (repeat ' ')
+                              ls' | length ls > 1 = map (" "++) ls
+                                  | otherwise     = ls
+                          in trace (s++":"++nlorspace++intercalate "\n" ls') a
 
--- | Print a showable value to the console, with a message, if the
--- debug level is at or above the specified level (uses
--- unsafePerformIO).
--- Values are displayed with pprint. Field names are not shown, but the
--- output is compact with smart line wrapping, long data elided,
--- and slow calculations timed out.
-dbgpprint :: Data a => Int -> String -> a -> a
-dbgpprint level msg a
-    | debugLevel >= level = unsafePerformIO $ do
-                              pprint a >>= putStrLn . ((msg++": \n") ++) . show
-                              return a
-    | otherwise           = a
+-- -- | Print a showable value to the console, with a message, if the
+-- -- debug level is at or above the specified level (uses
+-- -- unsafePerformIO).
+-- -- Values are displayed with pprint. Field names are not shown, but the
+-- -- output is compact with smart line wrapping, long data elided,
+-- -- and slow calculations timed out.
+-- dbgpprint :: Data a => Int -> String -> a -> a
+-- dbgpprint level msg a
+--     | debugLevel >= level = unsafePerformIO $ do
+--                               pprint a >>= putStrLn . ((msg++": \n") ++) . show
+--                               return a
+--     | otherwise           = a
 
 
 -- | Like dbg, then exit the program. Uses unsafePerformIO.
